==> cloud-3.0.0-rc1.tar.gz: cloud/modules/cloud_service_providers/openstack/src/Form/OpenStackSecurityGroupEditForm.php <==
<?php

namespace Drupal\openstack\Form;

use Drupal\openstack\Service\Ec2\OpenStackService as OpenStackEc2Service;
use Drupal\openstack\Service\OpenStackServiceFactoryInterface;
use Drupal\aws_cloud\Form\Ec2\SecurityGroupEditForm;
use Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface;
use Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface;
use Drupal\cloud\Service\EntityLinkRendererInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\Plugin\CachedDiscoveryClearerInterface;
use Drupal\Core\Render\Renderer;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form controller for the SecurityGroup entity edit form.
 *
 * @ingroup openstack
 */
class OpenStackSecurityGroupEditForm extends SecurityGroupEditForm {

  /**
   * The OpenStack Service Factory.
   *
   * @var \Drupal\openstack\Service\OpenStackServiceFactoryInterface
   */
  protected $openStackServiceFactory;

  /**
   * OpenStackSecurityGroupEditForm constructor.
   *
   * @param \Drupal\openstack\Service\OpenStackServiceFactoryInterface $openstack_service_factory
   *   Object for interfacing with OpenStack Service.
   * @param \Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface $ec2_service
   *   The AWS Cloud or OpenStack EC2 Service.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository
   *   The entity repository service.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   The entity type bundle service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   The Messenger Service.
   * @param \Drupal\cloud\Service\EntityLinkRendererInterface $entity_link_renderer
   *   The entity link render service.
   * @param \Drupal\Core\Entity\EntityTypeManager $entity_type_manager
   *   The Entity Type Manager.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cacheRender
   *   A cache backend interface instance.
   * @param \Drupal\Core\Plugin\CachedDiscoveryClearerInterface $plugin_cache_clearer
   *   A plugin cache clear instance.
   * @param \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface $cloud_config_plugin_manager
   *   The cloud service provider plugin manager (CloudConfigPluginManager).
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter.
   * @param \Drupal\Core\Render\Renderer $renderer
   *   The general renderer.
   */
  public function __construct(OpenStackServiceFactoryInterface $openstack_service_factory,
                              Ec2ServiceInterface $ec2_service,
                              EntityRepositoryInterface $entity_repository,
                              EntityTypeBundleInfoInterface $entity_type_bundle_info,
                              TimeInterface $time,
                              Messenger $messenger,
                              EntityLinkRendererInterface $entity_link_renderer,
                              EntityTypeManager $entity_type_manager,
                              CacheBackendInterface $cacheRender,
                              CachedDiscoveryClearerInterface $plugin_cache_clearer,
                              CloudConfigPluginManagerInterface $cloud_config_plugin_manager,
                              AccountInterface $current_user,
                              RouteMatchInterface $route_match,
                              DateFormatterInterface $date_formatter,
                              Renderer $renderer) {

    parent::__construct(
      $ec2_service,
      $entity_repository,
      $entity_type_bundle_info,
      $time,
      $messenger,
      $entity_link_renderer,
      $entity_type_manager,
      $cacheRender,
      $plugin_cache_clearer,
      $cloud_config_plugin_manager,
      $current_user,
      $route_match,
      $date_formatter,
      $renderer
    );

    $this->openStackServiceFactory = $openstack_service_factory;
  }

  /**
   * Dependency Injection.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   Instance of ContainerInterface.
   *
   * @return OpenStackSecurityGroupEditForm
   *   return created object.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('openstack.factory'),
      $container->get('openstack.ec2'),
      $container->get('entity.repository'),
      $container->get('entity_type.bundle.info'),
      $container->get('datetime.time'),
      $container->get('messenger'),
      $container->get('entity.link_renderer'),
      $container->get('entity_type.manager'),
      $container->get('cache.render'),
      $container->get('plugin.cache_clearer'),
      $container->get('plugin.manager.cloud_config_plugin'),
      $container->get('current_user'),
      $container->get('current_route_match'),
      $container->get('date.formatter'),
      $container->get('renderer')
    );
  }

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::buildForm().
   *
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   * @param string $cloud_context
   *   A cloud_context string value from URL "path".
   *
   * @return array
   *   Array of form object.
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cloud_context = '') {

    $form = parent::buildForm($form, $form_state, $cloud_context);

    $openstack_ec2_service_type = $this->openStackServiceFactory->isEc2ServiceType('cloud_config', $cloud_context);

    if (!empty($this->entity->getVpcId()) || empty($openstack_ec2_service_type)) {
      $form['rules'][] = $form['outbound_permission'];
    }
    unset($form['ip_permission'], $form['outbound_permission']);
    return $form;
  }

  /**
   * Submit handler.
   *
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   */
  public function save(array $form, FormStateInterface $form_state) {
    $entity = $this->entity;

    // Switch OpenStack EC2 or REST service based on $cloud_context.
    $this->ec2Service = $this->openStackServiceFactory->get($entity->getCloudContext());
    if ($this->ec2Service instanceof OpenStackEc2Service) {
      parent::save($form, $form_state);
      return;
    }
    // Call copyFormItemValues() to ensure the form array is intact.
    $this->copyFormItemValues($form);

    $this->trimTextfields($form, $form_state);

    $this->ec2Service->setCloudContext($entity->getCloudContext());

    // Fetch the most up to date security group data from Ec2.
    $existing_group = $this->ec2Service->describeSecurityGroups([
      'id' => $entity->getGroupId(),
    ]);

    if (!empty($existing_group)) {

      // Update the inbound/outbound permissions.
      $existing_permissions_inbound = $existing_group['SecurityGroups'][0]['IpPermissions'] ?? [];
      // Clear out the existing permissions.
      foreach ($existing_permissions_inbound ?: [] as $key => $inbound_permissions_list) {
        $ip_param_array = [
          'RuleId' => $inbound_permissions_list['RuleId'],
        ];
        $this->ec2Service->revokeSecurityGroupRule($ip_param_array);
      }

      // Update the inbound/outbound permissions.
      $existing_permissions_outbound = $existing_group['SecurityGroups'][0]['IpPermissionsEgress'] ?? [];
      // Clear out the existing permissions.
      foreach ($existing_permissions_outbound ?: [] as $key => $outbound_permissions_list) {
        $ip_param_array = [
          'RuleId' => $outbound_permissions_list['RuleId'],
        ];
        $this->ec2Service->revokeSecurityGroupRule($ip_param_array);
      }

      // Have the system refresh the security group.
      $this->ec2Service->updateSecurityGroups([
        'GroupIds' => [$this->entity->getGroupId()],
      ], FALSE);

      $this->clearCacheValues();
    }
    else {
      $this->messenger->addError($this->t('Cannot update security group.'));
    }

    if ($entity->save()) {

      $ip_perm_arr = ($entity->get('ip_permission') !== NULL)
        ? $entity->get('ip_permission')->getValue() : [];
      $outbound_perm_arr = ($entity->get('outbound_permission') !== NULL)
        ? $entity->get('outbound_permission')->getValue() : [];

      $this->ec2Service->updateSecurityGroup([
        'GroupId' => $entity->getGroupId(),
        'GroupName' => $entity->getName(),
      ]);

      // Create inbound security group rule.
      foreach ($ip_perm_arr ?: [] as $ip_perm) {
        if (!empty($ip_perm)) {
          $inbound_rule_param = [
            'Direction' => 'ingress',
            'FromPort' => $ip_perm['from_port'],
            'ToPort' => $ip_perm['to_port'],
            'IpProtocol' => $ip_perm['ip_protocol'],
            'GroupId' => $entity->getGroupId(),
            'Description' => $ip_perm['description'],
          ];

          switch ($ip_perm['source']) {
            case 'ip4':
              $inbound_rule_param += [
                'Source' => 'IPv4',
                'CidrIp' => $ip_perm['cidr_ip'],
              ];
              $inbound_rule_result = $this->ec2Service->createSecurityGroupRule($inbound_rule_param);
              $ip_perm['rule_id'] = $inbound_rule_result['RuleId'];
              break;

            case 'group':
              $inbound_rule_param += [
                'Source' => 'IPv4',
                'RemoteGroupId' => $ip_perm['group_id'],
                'RemoteUserId' => $ip_perm['user_id'],
              ];
              $group_inbound_rule_result = $this->ec2Service->createSecurityGroupRule($inbound_rule_param);
              $ip_perm['rule_id'] = $group_inbound_rule_result['RuleId'];
              break;

            case 'ip6':
              $inbound_rule_param += [
                'Source' => 'IPv6',
                'CidrIpv6' => $ip_perm['cidr_ip_v6'],
              ];
              $ip_six_inbound_rule_result = $this->ec2Service->createSecurityGroupRule($inbound_rule_param);
              $ip_perm['rule_id'] = $ip_six_inbound_rule_result['RuleId'];
              break;
          }
          $entity->set('ip_permission', $ip_perm);
        }
      }

      // Create outbound security group rule.
      foreach ($outbound_perm_arr ?: [] as $outbound_perm) {
        if (!empty($outbound_perm)) {
          $outbound_rule_param = [
            'Direction' => 'egress',
            'FromPort' => $outbound_perm['from_port'],
            'ToPort' => $outbound_perm['to_port'],
            'IpProtocol' => $outbound_perm['ip_protocol'],
            'GroupId' => $entity->getGroupId(),
            'Description' => $outbound_perm['description'],
          ];

          switch ($outbound_perm['source']) {
            case 'ip4':
              $outbound_rule_param += [
                'Source' => 'IPv4',
                'CidrIp' => $outbound_perm['cidr_ip'],
              ];
              $outbound_rule_result = $this->ec2Service->createSecurityGroupRule($outbound_rule_param);
              $outbound_perm['rule_id'] = $outbound_rule_result['RuleId'];
              break;

            case 'group':
              $outbound_rule_param += [
                'Source' => 'IPv4',
                'RemoteGroupId' => $outbound_perm['group_id'],
                'RemoteUserId' => $outbound_perm['user_id'],
              ];
              $group_outbound_rule_result = $this->ec2Service->createSecurityGroupRule($outbound_rule_param);
              $outbound_perm['rule_id'] = $group_outbound_rule_result['RuleId'];
              break;

            case 'ip6':
              $outbound_rule_param += [
                'Source' => 'IPv6',
                'CidrIpv6' => $outbound_perm['cidr_ip_v6'],
              ];
              $ip_six_outbound_rule_result = $this->ec2Service->createSecurityGroupRule($outbound_rule_param);
              $outbound_perm['rule_id'] = $ip_six_outbound_rule_result['RuleId'];
              break;
          }
          $entity->set('outbound_permission', $outbound_perm);
        }
      }

      // Have the system refresh the security group.
      $this->ec2Service->updateSecurityGroups([
        'GroupIds' => [$entity->getGroupId()],
      ], FALSE);

      if (count($this->messenger->messagesByType('error')) === 0) {
        $this->processOperationStatus($entity, 'updated');
        $form_state->setRedirect("entity.{$entity->getEntityTypeId()}.canonical", [
          'cloud_context' => $entity->getCloudContext(),
          $entity->getEntityTypeId() => $entity->id(),
        ]);
      }
    }
    else {
      $this->processOperationErrorStatus($this->entity, 'updated');
    }
  }

}

==> cloud-3.2.0.tar.gz: cloud/modules/cloud_service_providers/openstack/src/Form/OpenStackSecurityGroupEditForm.php <==
<?php

namespace Drupal\openstack\Form;

use Drupal\openstack\Service\Ec2\OpenStackService as OpenStackEc2Service;
use Drupal\openstack\Service\OpenStackServiceFactoryInterface;
use Drupal\aws_cloud\Form\Ec2\SecurityGroupEditForm;
use Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface;
use Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface;
use Drupal\cloud\Service\EntityLinkRendererInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\Plugin\CachedDiscoveryClearerInterface;
use Drupal\Core\Render\Renderer;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form controller for the SecurityGroup entity edit form.
 *
 * @ingroup openstack
 */
class OpenStackSecurityGroupEditForm extends SecurityGroupEditForm {

  /**
   * The OpenStack Service Factory.
   *
   * @var \Drupal\openstack\Service\OpenStackServiceFactoryInterface
   */
  protected $openStackServiceFactory;

  /**
   * OpenStackSecurityGroupEditForm constructor.
   *
   * @param \Drupal\openstack\Service\OpenStackServiceFactoryInterface $openstack_service_factory
   *   Object for interfacing with OpenStack Service.
   * @param \Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface $ec2_service
   *   The AWS Cloud or OpenStack EC2 Service.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository
   *   The entity repository service.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   The entity type bundle service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   The Messenger Service.
   * @param \Drupal\cloud\Service\EntityLinkRendererInterface $entity_link_renderer
   *   The entity link render service.
   * @param \Drupal\Core\Entity\EntityTypeManager $entity_type_manager
   *   The Entity Type Manager.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cacheRender
   *   A cache backend interface instance.
   * @param \Drupal\Core\Plugin\CachedDiscoveryClearerInterface $plugin_cache_clearer
   *   A plugin cache clear instance.
   * @param \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface $cloud_config_plugin_manager
   *   The cloud service provider plugin manager (CloudConfigPluginManager).
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter.
   * @param \Drupal\Core\Render\Renderer $renderer
   *   The general renderer.
   */
  public function __construct(OpenStackServiceFactoryInterface $openstack_service_factory,
                              Ec2ServiceInterface $ec2_service,
                              EntityRepositoryInterface $entity_repository,
                              EntityTypeBundleInfoInterface $entity_type_bundle_info,
                              TimeInterface $time,
                              Messenger $messenger,
                              EntityLinkRendererInterface $entity_link_renderer,
                              EntityTypeManager $entity_type_manager,
                              CacheBackendInterface $cacheRender,
                              CachedDiscoveryClearerInterface $plugin_cache_clearer,
                              CloudConfigPluginManagerInterface $cloud_config_plugin_manager,
                              AccountInterface $current_user,
                              RouteMatchInterface $route_match,
                              DateFormatterInterface $date_formatter,
                              Renderer $renderer) {

    parent::__construct(
      $ec2_service,
      $entity_repository,
      $entity_type_bundle_info,
      $time,
      $messenger,
      $entity_link_renderer,
      $entity_type_manager,
      $cacheRender,
      $plugin_cache_clearer,
      $cloud_config_plugin_manager,
      $current_user,
      $route_match,
      $date_formatter,
      $renderer
    );

    $this->openStackServiceFactory = $openstack_service_factory;
  }

  /**
   * Dependency Injection.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   Instance of ContainerInterface.
   *
   * @return OpenStackSecurityGroupEditForm
   *   return created object.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('openstack.factory'),
      $container->get('openstack.ec2'),
      $container->get('entity.repository'),
      $container->get('entity_type.bundle.info'),
      $container->get('datetime.time'),
      $container->get('messenger'),
      $container->get('entity.link_renderer'),
      $container->get('entity_type.manager'),
      $container->get('cache.render'),
      $container->get('plugin.cache_clearer'),
      $container->get('plugin.manager.cloud_config_plugin'),
      $container->get('current_user'),
      $container->get('current_route_match'),
      $container->get('date.formatter'),
      $container->get('renderer')
    );
  }

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::buildForm().
   *
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   * @param string $cloud_context
   *   A cloud_context string value from URL "path".
   *
   * @return array
   *   Array of form object.
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cloud_context = ''): array {

    $form = parent::buildForm($form, $form_state, $cloud_context);

    $openstack_ec2_service_type = $this->openStackServiceFactory->isEc2ServiceType('cloud_config', $cloud_context);

    if (!empty($this->entity->getVpcId()) || empty($openstack_ec2_service_type)) {
      $form['rules'][] = $form['outbound_permission'];
    }
    unset($form['ip_permission'], $form['outbound_permission']);
    return $form;
  }

  /**
   * Submit handler.
   *
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   */
  public function save(array $form, FormStateInterface $form_state): void {
    $entity = $this->entity;

    // Switch OpenStack EC2 or REST service based on $cloud_context.
    $this->ec2Service = $this->openStackServiceFactory->get($entity->getCloudContext());
    if ($this->ec2Service instanceof OpenStackEc2Service) {
      parent::save($form, $form_state);
      return;
    }
    // Call copyFormItemValues() to ensure the form array is intact.
    $this->copyFormItemValues($form);

    $this->trimTextfields($form, $form_state);

    $this->ec2Service->setCloudContext($entity->getCloudContext());

    // Fetch the most up to date security group data from Ec2.
    $existing_group = $this->ec2Service->describeSecurityGroups([
      'id' => $entity->getGroupId(),
    ]);

    if (!empty($existing_group)) {

      // Update the inbound/outbound permissions.
      $existing_permissions_inbound = $existing_group['SecurityGroups'][0]['IpPermissions'] ?? [];
      // Clear out the existing permissions.
      foreach ($existing_permissions_inbound ?: [] as $inbound_permissions_list) {
        $ip_param_array = [
          'RuleId' => $inbound_permissions_list['RuleId'],
        ];
        $this->ec2Service->revokeSecurityGroupRule($ip_param_array);
      }

      // Update the inbound/outbound permissions.
      $existing_permissions_outbound = $existing_group['SecurityGroups'][0]['IpPermissionsEgress'] ?? [];
      // Clear out the existing permissions.
      foreach ($existing_permissions_outbound ?: [] as $outbound_permissions_list) {
        $ip_param_array = [
          'RuleId' => $outbound_permissions_list['RuleId'],
        ];
        $this->ec2Service->revokeSecurityGroupRule($ip_param_array);
      }

      // Have the system refresh the security group.
      $this->ec2Service->updateSecurityGroups([
        'GroupIds' => [$this->entity->getGroupId()],
      ], FALSE);

      $this->clearCacheValues();
    }
    else {
      $this->messenger->addError($this->t('Cannot update security group.'));
    }

    if ($entity->save()) {

      $ip_perm_arr = ($entity->get('ip_permission') !== NULL)
        ? $entity->get('ip_permission')->getValue() : [];
      $outbound_perm_arr = ($entity->get('outbound_permission') !== NULL)
        ? $entity->get('outbound_permission')->getValue() : [];

      $this->ec2Service->updateSecurityGroup([
        'GroupId' => $entity->getGroupId(),
        'GroupName' => $entity->getName(),
      ]);

      // Create inbound security group rule.
      foreach ($ip_perm_arr ?: [] as $ip_perm) {
        if (!empty($ip_perm)) {
          $inbound_rule_param = [
            'Direction' => 'ingress',
            'FromPort' => $ip_perm['from_port'],
            'ToPort' => $ip_perm['to_port'],
            'IpProtocol' => $ip_perm['ip_protocol'],
            'GroupId' => $entity->getGroupId(),
            'Description' => $ip_perm['description'],
          ];

          switch ($ip_perm['source']) {
            case 'ip4':
              $inbound_rule_param += [
                'Source' => 'IPv4',
                'CidrIp' => $ip_perm['cidr_ip'],
              ];
              $inbound_rule_result = $this->ec2Service->createSecurityGroupRule($inbound_rule_param);
              $ip_perm['rule_id'] = $inbound_rule_result['RuleId'];
              break;

            case 'group':
              $inbound_rule_param += [
                'Source' => 'IPv4',
                'RemoteGroupId' => $ip_perm['group_id'],
                'RemoteUserId' => $ip_perm['user_id'],
              ];
              $group_inbound_rule_result = $this->ec2Service->createSecurityGroupRule($inbound_rule_param);
              $ip_perm['rule_id'] = $group_inbound_rule_result['RuleId'];
              break;

            case 'ip6':
              $inbound_rule_param += [
                'Source' => 'IPv6',
                'CidrIpv6' => $ip_perm['cidr_ip_v6'],
              ];
              $ip_six_inbound_rule_result = $this->ec2Service->createSecurityGroupRule($inbound_rule_param);
              $ip_perm['rule_id'] = $ip_six_inbound_rule_result['RuleId'];
              break;
          }
          $entity->set('ip_permission', $ip_perm);
        }
      }

      // Create outbound security group rule.
      foreach ($outbound_perm_arr ?: [] as $outbound_perm) {
        if (!empty($outbound_perm)) {
          $outbound_rule_param = [
            'Direction' => 'egress',
            'FromPort' => $outbound_perm['from_port'],
            'ToPort' => $outbound_perm['to_port'],
            'IpProtocol' => $outbound_perm['ip_protocol'],
            'GroupId' => $entity->getGroupId(),
            'Description' => $outbound_perm['description'],
          ];

          switch ($outbound_perm['source']) {
            case 'ip4':
              $outbound_rule_param += [
                'Source' => 'IPv4',
                'CidrIp' => $outbound_perm['cidr_ip'],
              ];
              $outbound_rule_result = $this->ec2Service->createSecurityGroupRule($outbound_rule_param);
              $outbound_perm['rule_id'] = $outbound_rule_result['RuleId'];
              break;

            case 'group':
              $outbound_rule_param += [
                'Source' => 'IPv4',
                'RemoteGroupId' => $outbound_perm['group_id'],
                'RemoteUserId' => $outbound_perm['user_id'],
              ];
              $group_outbound_rule_result = $this->ec2Service->createSecurityGroupRule($outbound_rule_param);
              $outbound_perm['rule_id'] = $group_outbound_rule_result['RuleId'];
              break;

            case 'ip6':
              $outbound_rule_param += [
                'Source' => 'IPv6',
                'CidrIpv6' => $outbound_perm['cidr_ip_v6'],
              ];
              $ip_six_outbound_rule_result = $this->ec2Service->createSecurityGroupRule($outbound_rule_param);
              $outbound_perm['rule_id'] = $ip_six_outbound_rule_result['RuleId'];
              break;
          }
          $entity->set('outbound_permission', $outbound_perm);
        }
      }

      // Have the system refresh the security group.
      $this->ec2Service->updateSecurityGroups([
        'GroupIds' => [$entity->getGroupId()],
      ], FALSE);

      if (count($this->messenger->messagesByType('error')) === 0) {
        $this->processOperationStatus($entity, 'updated');
        $form_state->setRedirect("entity.{$entity->getEntityTypeId()}.canonical", [
          'cloud_context' => $entity->getCloudContext(),
          $entity->getEntityTypeId() => $entity->id(),
        ]);
      }
    }
    else {
      $this->processOperationErrorStatus($this->entity, 'updated');
    }
  }

}
